/*
 *	This plugin changes the amount of resources aliens receive from RTs so that it is independent of the number of players.
 *	Good for servers with a large max players.
 *
 *	How it works:
 *
 *	Normally NS gives 1 res per RT every 4 seconds split up among all aliens. The algorithm looks like this:
 *		PlayerResIncrease = NumResTowers / NumAlienPlayers
 *
 *	This is fine if you have a 12-14 player server, otherwise it becomes unbalanced as aliens receive too little or too much res.
 *
 *	This plugin changes it so that its:
 *		PlayerResIncrease = NumResTowers / 7
 *
 *	You can change the default from 7 with the cvar "amx_rb_baseres". I picked 7 as that is what seems most balanced for our server.
 *
 *
 *	Note:		The plugin automatically disables itself when tournament mode is enabled and was designed for and tested in NS3b4a.
 *
 *	Commands:	None
 *
 *	Cvars:		amx_rb_baseres
 *				This controls how much res the aliens receive. 7 means its the same res as if it was a 7v7 game (default 7)
 *
 *			amx_rb_increaseonly
 *				If set to 1 alien resources will only be increased, they won't be reduced. (default 0)
 *
 *    
 *	Requires:	ns2amx Module 1.0
 *
 *	Author:		Cheesy Peteza  
 *	Date:		7-June-2004
 *	Email:		cheesy@yoclan.com
 *	irc:		#yo-clan (QuakeNet)
 *
 */

#include <amxmod>
#include <ns2amx>

#define ALIEN			2
#define PRIVATE_ACTIVE_RES	27	// Alien res chamber offset that specifies whether its active or not.
					// (There appears to be an approx 5 second activation time after it is built)
new g_maxPlayers
new Float:g_lastres[33]
new Float:g_numrt, Float:g_defaultres

public plugin_init() {
	register_plugin("Resource Balancer", "1.1", "Cheesy Peteza")
	register_cvar("resbalancer_version", "1.1", FCVAR_SERVER)
	register_cvar("amx_rb_baseres", "7", FCVAR_SERVER)
	register_cvar("amx_rb_increaseonly", "0", FCVAR_SERVER)

	g_maxPlayers = get_maxplayers()
}

/* This long winded method is here so that I can alter players resources at the exact same time NS does for res chambers.
   Yes it sucks but I do not know a better way of doing it that doesn't cause HUD glitches or allow exploiting.
   It may seem a bit overly complicated but this is so that I do as little work as possible as this gets runs every server frame */
public server_frame() {
	for(new id = 1; id <= g_maxPlayers; ++id) {		// 99.9% of the time res won't change so check for that first
		if ( pev(id, pev_team) != ALIEN ) continue	// to save the server some work
		if (get_private_f(id,PRIVATE_ALIEN_RES) == g_lastres[id]) return
	}

	new numaliens = get_num_aliens()
	g_numrt = float( get_num_active_rt() )
	if (g_numrt == 0.0) return

	g_defaultres = g_numrt / numaliens	// This is how much res NS will increase per tick.

	for(new id = 1; id <= g_maxPlayers; ++id) {
		if ( pev(id, pev_team) != ALIEN ) continue

		new Float:res = get_private_f(id,PRIVATE_ALIEN_RES)
		new Float:testres = res - g_lastres[id] - g_defaultres	// If this comes to 0 it means that NS just increased a
		g_lastres[id] = res					// players res due to resource chamber res distribution.

		if (-0.0005 < testres < 0.0005) {	// Allow for some slight error
			balanceRes()			// NS is updating res, lets make our modifications at the same time so
			break				// that players don't notice any difference.
		}
	}
}

/* This should be run every 4 seconds, when aliens are receiving the res from the resource chambers */
public balanceRes() {
	if ( get_cvar_num("mp_tournamentmode") ) return PLUGIN_HANDLED

	new Float:baseres = get_cvar_float("amx_rb_baseres")
	if ( get_cvar_num("amx_rb_increaseonly") )
		if (get_num_aliens() <= baseres) return PLUGIN_HANDLED

	new Float:newres = g_numrt / baseres			// This is how much we really want to give
	new Float:resmodifier = newres - g_defaultres		// This is the difference between what we want and NS wants

	for(new id = 1; id <= g_maxPlayers; ++id) {
		if ( pev(id, pev_team) != ALIEN ) continue

		new Float:res = get_private_f(id,PRIVATE_ALIEN_RES)
		res += resmodifier				// Add on the difference
		if (res < 0.0) res = 0.0
		set_private_f(id, PRIVATE_ALIEN_RES, res)
		g_lastres[id] = res
	}

	return PLUGIN_HANDLED
}

get_num_aliens() {
	new num
	for(new id = 1; id <= g_maxPlayers; ++id) {
		if ( pev(id, pev_team) != ALIEN ) continue
		num++
	}
	return num
}

get_num_active_rt() {
	new numrt = get_ent("alienresourcetower", 0)
	new num

	for (new i = 1; i <= numrt; ++i) {
		new id = get_ent("alienresourcetower", i)
		if ( get_private_i(id, PRIVATE_ACTIVE_RES, 4) )	// Resource towers have an activation time of about 5 secs after
			num++					// they are built before they start giving res out.
	}
	return num
}
